classdef shape < handle
    %SHAPE Abstract superclass for all shapes.
    %   Provides a generic no-arg constructor, which calls a
    %   make_shape method that must be defined for subclasses.
    %   Prepares the transform used to control visibility, 
    %   translation (centre) and rotation (angle).
    
    properties
        Transform;
    end
    
    methods (Abstract)
        make_shape(self);
        colour(new_colour);
        color(new_color);
    end
        
    methods
        
        function self = shape
            self.Transform = hgtransform(...
                'Parent', gca, ...
                'Matrix', makehgtform('translate', [500 500 0]));
            self.make_shape;
        end
        
        function rv = visible(self, show)
            error(nargchk(1, 2, nargin, 'struct'));
            if nargin == 2
                if show
                    set(self.Transform, 'Visible', 'on');
                else
                    set(self.Transform, 'Visible', 'off');
                end
                rv = self;
            else
                rv = strcmp(get(self.Transform, 'Visible'), 'on');
            end
        end
                           
        function rv = centre(self, new_centre)
           error(nargchk(1, 2, nargin, 'struct'));
           matrix = get(self.Transform, 'Matrix');
           t_centre = [matrix(1,4) matrix(2,4)]; 
           if nargin == 2
                displacement = new_centre - t_centre;
                tform = makehgtform('translate', horzcat(displacement, 0));
                set(self.Transform, 'Matrix', tform * matrix);
                rv = self;
           else 
                rv = t_centre;
           end
        end
        
        function rv = move(self, displacement)
           error(nargchk(1, 2, nargin, 'struct'));
           matrix = get(self.Transform, 'Matrix');
           if nargin == 2
               self.centre(self.centre + displacement);
               rv = self;
                % tform = makehgtform('translate', horzcat(displacement, 0));
                % set(self.Transform, 'Matrix', tform * matrix);
                % rv = self;
           else 
                rv = [0 0];
           end
        end
        
        function rv = angle(self, new_angle)
            error(nargchk(1, 2, nargin, 'struct'));
            matrix = get(self.Transform, 'Matrix');
            t_angle = angle(matrix(1,1) + 1i*matrix(2,1));
            if nargin == 2
                to = makehgtform('translate', horzcat(-self.centre, 0));
                rot = makehgtform('zrotate', new_angle - t_angle);
                from = makehgtform('translate', horzcat(self.centre, 0));
                set(self.Transform, 'Matrix', ...
                    from *rot * to * matrix);
                rv = self;
            else
                rv = t_angle;
            end
        end
        
        function rv = rotate(self, rotation_angle)
            error(nargchk(1, 2, nargin, 'struct'));
            if nargin == 2
                self.angle(self.angle + rotation_angle);
            else
                rv = 0;
            end
        end
    end
                
end

% Copyright (c) Greg Phillips, 2010
% 8 November 2010
%
% This file is part of MATLAB-GRAPHICS.
% 
% MATLAB-GRAPHICS is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% MATLAB-GRAPHICS is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with MATLAB-GRAPHICS.  If not, see <http://www.gnu.org/licenses/>.



